-- ========================================================
-- SELECT base

-----------------------------------------------------------
-- 01) Trovare il codice e il numero di soci
--     dei fornitori di Milano
SELECT CodF, NSoci
FROM   F
WHERE  Sede = 'Milano';

-----------------------------------------------------------
 -- 02) Trovare il codice di tutti i prodotti
SELECT CodP
FROM   P;

-----------------------------------------------------------
-- 03) Trovare il codice dei prodotti (diversi)
--     forniti da almeno un fornitore

--     a) Con duplicazione dei risultati
SELECT CodP
FROM   FP;

--     b) Con risultati univoci
SELECT DISTINCT CodP
FROM   FP;

-----------------------------------------------------------
-- 04) Trovare tutte le informazioni sui prodotti

--     a) Con indicazione esplicita di tutti gli attributi
SELECT CodP, NomeP, Colore, Taglia, Magazzino
FROM   P;

--     b) Con indicazione implicita di tutti gli attributi
SELECT *
FROM   P;

-----------------------------------------------------------
-- 05) Trovare il codice dei prodotti
--     e la taglia espressa con la misura americana...

--     a) senza denominazione colonna:
SELECT CodP, Taglia - 14
FROM   P;

--     b) con denominazione colonna (TagliaUSA):
SELECT CodP, Taglia - 14 AS TagliaUSA
FROM   P; 

-- ========================================================
-- Clausola WHERE

-----------------------------------------------------------
-- 01) Trovare il codice dei fornitori di Milano
SELECT CodF
FROM F
WHERE Sede = 'Milano';

-----------------------------------------------------------
-- 02) Trovare il codice e il numero di soci dei fornitori
--     che non hanno sede a Milano
SELECT CodF, NSoci 
FROM   F
WHERE  Sede <> 'Milano';

-----------------------------------------------------------
-- 03) Trovare il codice dei fornitori di Milano
--     con pi di 2 soci
SELECT CodF
FROM   F
WHERE  Sede = 'Milano'
       AND NSoci > 2;

-----------------------------------------------------------
-- 04) Trovare il codice e il numero di soci dei fornitori
--     di Milano oppure di Torino 
SELECT CodF, NSoci 
FROM   F
WHERE  Sede = 'Milano'
       OR Sede = 'Torino';
       
-----------------------------------------------------------
--     Trovare il codice e il numero di soci dei fornitori
--     di Milano E di Torino (???)
SELECT CodF, NSoci 
FROM   F
WHERE  Sede = 'Milano'
       AND Sede = 'Torino';
       
-----------------------------------------------------------
-- 05) Trovare il codice e il nome dei prodotti
--     il cui nome inizia con 'C'
SELECT CodP, NomeP
FROM   P
WHERE  NomeP LIKE 'C%';

-----------------------------------------------------------
-- 06) Trovare il codice e il nome dei fornitori
--     il cui nome contiene 'a' in seconda posizione
SELECT CodF, NomeF
FROM   F
WHERE  NomeF LIKE '_a%';

-----------------------------------------------------------
-- 07) Trovare il codice, il nome e la taglia dei prodotti
--     con taglia diversa da 44
SELECT CodP, NomeP, Taglia
FROM   P
WHERE  Taglia <> 44;

-----------------------------------------------------------
-- 08) Trovare il codice, il nome e la taglia
--     dei prodotti...

--     a) con taglia non nota:
SELECT CodP, NomeP, Taglia
FROM   P
WHERE  Taglia IS NULL;

--     b) con taglia nota:
SELECT CodP, NomeP, Taglia
FROM   P
WHERE  Taglia IS NOT NULL;


-- ========================================================
-- Ordinamento del risultato: ORDER BY

-----------------------------------------------------------
-- 01) Trovare tutti di dati dei prodotti
--     ordinando il risultato...

--     a) in ordine crescente di taglia
SELECT   *
FROM     P
ORDER BY Taglia;

--     b) in ordine decrescente di taglia
SELECT   *
FROM     P
ORDER BY Taglia DESC;

-----------------------------------------------------------
-- 02) Trovare tutti di dati dei prodotti
--     ordinando il risultato
--     in ordine crescente di magazzino
--     e in ordine decrescente di taglia
SELECT   *
FROM     P
ORDER BY Magazzino, Taglia DESC;

-----------------------------------------------------------
-- 03) Trovare il nome e la taglia USA di tutti i prodotti
--     ordinando il risultato
--     in ordine crescente di taglia USA
SELECT   NomeP, Taglia - 14 AS TagliaUSA
FROM     P
ORDER BY TagliaUSA;
--       Nota: taglia Usa della Gonna = NULL - 14 = NULL

-- ========================================================
-- Operazione di JOIN

-----------------------------------------------------------
-- 01) Trovare il codice dei prodotti
--     ed il nome dei fornitori che li forniscono

--     a) Prodotto cartesiano
SELECT CodP, NomeF 
FROM   F, FP;

--     b) Join con Prodotto cartesiano + selezione
SELECT CodP, NomeF 
FROM   F, FP
WHERE  F.CodF = FP.CodF;

--     c) Join con operatore INNER JOIN
SELECT CodP, NomeF 
FROM   F INNER JOIN FP ON F.CodF = FP.CodF;

-----------------------------------------------------------
-- 02) Trovare il nome dei fornitori
--     che forniscono il prodotto P2
SELECT NomeF 
FROM   F, FP
WHERE  F.CodF = FP.CodF
       AND CodP = 'P2';
--     Nota: gli attributi di ugual nome
--     devono essere 'qualificati' (F.CodF, FP.CodF)

-----------------------------------------------------------
-- 03) Trovare il nome dei fornitori 
--     ed il nome dei prodotti che forniscono

--     a) Join con Prodotto cartesiano + selezione
SELECT NomeF, NomeP
FROM   F, FP, P
WHERE  F.CodF = FP.CodF AND
       P.CodP = FP.CodP;
	   
--     b) Join con operatore INNER JOIN
SELECT NomeF, NomeP
FROM   F INNER JOIN FP ON F.CodF = FP.CodF
         INNER JOIN P ON P.CodP = FP.CodP;
       
-----------------------------------------------------------
-- 04) Trovare il nome dei fornitori
--     che forniscono almeno un prodotto rosso
SELECT DISTINCT NomeF
FROM   F, FP, P
WHERE  F.CodF = FP.CodF
       AND P.CodP = FP.CodP
       AND Colore='Rosso';
--     Nota: uno stesso fornitore pu fornire
--     pi prodotti rossi -> SELECT DISTINCT

-----------------------------------------------------------
-- 05) Trovare tutte le coppie di nomi di fornitori
--     che hanno sede nella stessa citt

--     a) Tutte le coppie (anche quelle inutli!)
SELECT FX.NomeF, FY.NomeF
FROM   F FX, F FY
WHERE  FX.Sede = FY.Sede;

--     b) Solo le coppie significative
SELECT FX.NomeF, FY.NomeF
FROM   F FX, F FY
WHERE  FX.Sede = FY.Sede
       AND FX.CodF < FY.CodF;
       
-----------------------------------------------------------
-- 06) Trovare il codice e il nome di TUTTI i fornitori
--     insieme al codice dei prodotti forniti (se esistono)
SELECT F.CodF, NomeF, CodP 
FROM   F LEFT OUTER JOIN FP ON F.CodF = FP.CodF;

-- ========================================================
-- Funzione aggregate: COUNT(), SUM(), AVG(), MIN(), MAX()

-----------------------------------------------------------
-- 01) Trovare il numero di fornitori
SELECT COUNT(*) 
FROM   F;

-----------------------------------------------------------
-- 02) Trovare il numero di fornitori
--     che hanno effettuato almeno una fornitura

--     a) Conteggio delle righe
SELECT COUNT(*)
FROM   FP;

--     b) Conteggio dei codici fornitore (anche se uguali)
SELECT COUNT(CodF)
FROM   FP;

--     c) Conteggio dei codici fornitore distinti
SELECT COUNT(DISTINCT CodF)
FROM   FP;

-----------------------------------------------------------
-- 03) Trovare il numero (con nome NumProdotti)...

--     a) di tutti i prodotti
SELECT COUNT(*) AS NumProdotti
       FROM P;
	   
--     b) dei prodotti di cui  nota la taglia
SELECT COUNT(Taglia) AS NumProdotti
       FROM P;

-----------------------------------------------------------
-- 04) Trovare il numero di fornitori
--     che forniscono il prodotto P2

--     a) selezione delle forniture di P2
SELECT *
FROM   FP
WHERE  CodP='P2';

--     b) conteggio delle forniture risultanti
SELECT COUNT(*) 
FROM   FP
    WHERE CodP = 'P2';
    
-----------------------------------------------------------
-- 05) Trovare la quantit totale di pezzi
--     forniti per il prodotto P2

--     a) selezione delle forniture di P2
SELECT *
FROM   FP
WHERE  CodP='P2';

--     b) somma delle forniture risultanti
SELECT SUM(Qta) 
FROM   FP
WHERE  CodP='P2';

-- ========================================================
-- Operatore GROUP BY

-----------------------------------------------------------
-- 01) Per OGNI PRODOTTO,
--     trovare il codice e la quantit totale
--     di pezzi forniti

--     a) Prodotti 'idealmente' raggruppati
SELECT   CodP, Qta 
FROM     FP
ORDER BY CodP;

--     b) Raggruppamento effettivo e somma di Qta
SELECT   CodP, SUM(Qta) 
FROM     FP
GROUP BY CodP;

-----------------------------------------------------------
-- 02) Per OGNI FORNITORE,
--     trovare il nome,
--     il numero totale di forniture
--     e la quantit totale di pezzi forniti
SELECT   CodF, COUNT(*), SUM(Qta) 
FROM     FP
GROUP BY CodF;

-----------------------------------------------------------
-- 03) Per OGNI FORNITORE,
--     trovare il nome,
--     il numero totale di forniture
--     la quantit totale di pezzi forniti
--     e il numero medio di pezzi per fornitura...

--     a) con il calcolo esplicito della media
SELECT   CodF, COUNT(*), SUM(Qta),
         SUM(Qta) / COUNT(*) AS Media
FROM     FP
GROUP BY CodF;

--     b) con la funzione AVG()
SELECT   CodF, COUNT(*), SUM(Qta),
         AVG(Qta) AS Media
FROM     FP
GROUP BY CodF;

-----------------------------------------------------------
-- 04) Per OGNI PRODOTTO,
--     trovare il nome, il colore, la taglia
--     e la quantit totale di pezzi forniti
SELECT   NomeP, Colore, Taglia, SUM(Qta) AS TotPezzi
FROM     P INNER JOIN FP ON P.CodP = FP.CodP
GROUP BY NomeP, Colore, Taglia;
--     -> gli attributi non soggetti ad aggregazione
--        (NomeP, Colore, Taglia)
--        devono essere compresi nella clausola GROUP BY

-----------------------------------------------------------
-- 05) Per OGNI PRODOTTO,
--     trovare la quantit totale di pezzi forniti
--     da fornitori con sede a Milano

--     a) Selezione (implicita) delle forniture
--        effettuate da fornitori con sede a Milano
SELECT   CodP, Qta
FROM     FP, F
WHERE    FP.CodF = F.CodF
         AND Sede = 'Milano';
		 
--     b) Raggruppamento del risultato
SELECT   CodP, SUM(Qta)
FROM     FP, F
WHERE    FP.CodF = F.CodF
         AND Sede = 'Milano'
GROUP BY CodP;

-----------------------------------------------------------
-- 06) Trovare il codice dei prodotti
--     e la quantit totale di pezzi forniti
--     per i prodotti per cui sono forniti in totale
--     almeno 600 pezzi
SELECT   CodP, SUM(Qta) 
FROM     FP
GROUP BY CodP
HAVING   SUM(Qta) >= 600;
-- HAVING specifica una condizione su funzioni aggregate

-----------------------------------------------------------
-- 07) Trovare il codice dei prodotti rossi
--     forniti da pi di un fornitore
SELECT   FP.CodP 
FROM     FP, P
WHERE    FP.CodP = P.CodP
         AND Colore = 'Rosso'
GROUP BY FP.CodP
HAVING   COUNT(*) > 1;

-----------------------------------------------------------
-- SELECT: ordine (rigoroso) delle clausole:
--    SELECT [DISTINCT] Attributi
--    FROM Tabelle
--    [WHERE CondizioniDiTupla]
--    [GROUP BY AttributiDiRaggruppamento]
--    [HAVING CondizioniSuAggregati] 
--    [ORDER BY AttributiDiOrdinamento];

