-- ========================================================
-- Concetti introduttivi

-----------------------------------------------------------
-- 01) Trovare il codice dei fornitori
--     che hanno sede nella stessa citt di F1

--     a) Ricerca della sede di F1
SELECT Sede
FROM   F
WHERE  CodF = 'F1';

--     b) Uso dei risultato all'interno di un'altra query
SELECT CodF 
FROM   F
WHERE  Sede = (
          SELECT Sede 
          FROM   F
          WHERE  CodF = 'F1'
        );

-----------------------------------------------------------
-- 02) Trovare il codice dei fornitori
--     che hanno il massimo numero di soci

--     a) Ricerca del numero massimo di soci
SELECT MAX(NSoci) 
       FROM F;

--     b) Uso dei risultato all'interno di un'altra query
SELECT CodF 
FROM   F
WHERE  NSoci = (
          SELECT MAX(NSoci) 
          FROM   F
       );
       
-----------------------------------------------------------
-- 03) Trovare il codice dei prodotti che hanno
--     il maggior numero totale di pezzi forniti

--     a) Ricerca del numero totale di pezzi forniti 
--        per ogni prodotto (TotForniture)
SELECT   CodP, SUM(Qta) AS TotForniture
FROM     FP
GROUP BY CodP;

--     b) Ricerca del massimo di TotForniture (= 800)
SELECT   MAX(TotForniture)
FROM     (
            SELECT   CodP, SUM(Qta) AS TotForniture
            FROM     FP
            GROUP BY CodP
         );
         
--     c) Uso dei risultato all'interno di un'altra query
--        (ricerca del codice dei prodotti che hanno
--        un totale di 800 pezzi forniti)
SELECT   CodP, SUM(Qta)
FROM     FP
GROUP BY CodP
HAVING   SUM(Qta) = (
            SELECT   MAX(TotForniture)
            FROM (
               SELECT   CodP, SUM(Qta) AS TotForniture
               FROM     FP
               GROUP BY CodP
            )
         );
         
-- ========================================================
-- Operatore IN 

-----------------------------------------------------------
-- 01) Trovare il nome dei fornitori
--     che hanno uno dei codici 'F1', 'F3', 'F5'
SELECT NomeF
FROM   F
WHERE  CodF IN ('F1', 'F3', 'F5');
-- l'operatore IN si applica ad una LISTA di valori;
--  l'estensione dell'operatore =

----------------------------------------------------------
-- 02) Trovare il codice dei fornitori
--     che forniscono almeno un prodotto rosso

--     a) Produzione della lista dei codici dei prodotti rossi
--        Risultato: ('P1, 'P6')
SELECT CodP
FROM   P
WHERE  Colore = 'Rosso';

--     b) Ricerca delle forniture in cui il codice prodotto
--        sta in ('P1, 'P6')
SELECT DISTINCT CodF
FROM   FP
WHERE  CodP IN (
          SELECT CodP
          FROM   P
          WHERE  Colore = 'Rosso'
       );
-- Nota: il risultato si pu ottenere anche con un JOIN

-----------------------------------------------------------
-- 03) Trovare il codice dei fornitori
--     che forniscono almeno uno dei prodotti
--     forniti da F2

--     a) Produzione della lista dei codici dei prodotti
--        forniti da F2
--        Risultato: ('P1, 'P2')
SELECT CodP
FROM   FP
WHERE  CodF = 'F2';

--     b) Ricerca delle forniture con fornitore diverso da F2
--        in cui il codice prodotto sta in ('P1, 'P2') 
SELECT DISTINCT CodF
FROM   FP
WHERE  CodF <> 'F2'
       AND CodP IN (
          SELECT CodP
          FROM   FP
          WHERE  CodF = 'F2'
       );
-- Nota: il risultato si pu ottenere anche con un JOIN
         
-- ========================================================
-- Operatore NOT IN

-----------------------------------------------------------
-- 01) Trovare il nome dei fornitori
--     che forniscono almeno un prodotto diverso da P2
SELECT DISTINCT NomeF 
FROM   F, FP
WHERE  F.CodF=FP.CodF 
       AND CodP <> 'P2';

-----------------------------------------------------------
-- 02) Trovare il nome dei fornitori
--     che NON forniscono il prodotto P2

--     a) Ricerca dei codici dei fornitori che forniscono P2
--        Risultato: ('F1', 'F2', 'F3')
SELECT CodF
FROM   FP
WHERE  CodP = 'P2';

--     b) Ricerca dei fornitori 
--        il cui codice NON sta in ('F1', 'F2', 'F3')
SELECT NomeF
FROM   F
WHERE  CodF NOT IN (
          SELECT CodF
          FROM   FP
          WHERE  CodP = 'P2'
       );
       
-----------------------------------------------------------
-- 03) Trovare il nome dei fornitori
--     che NON forniscono prodotti diversi da P2
--     (possono anche non fornire alcun prodotto)

--     a) Ricerca dei codici dei fornitori che forniscono
--        prodotti diversi da P2
--        Risultato: ('F1', 'F2', 'F4')
SELECT CodF
FROM   FP
WHERE  CodP <> 'P2';

--     b) Ricerca dei fornitori 
--        il cui codice NON sta in ('F1', 'F2', 'F4')
SELECT NomeF
FROM   F
WHERE  CodF NOT IN (
          SELECT CodF
          FROM   FP
          WHERE  CodP <> 'P2'
       );
       
-----------------------------------------------------------
-- 03) Trovare il nome dei fornitori
--     che forniscono SOLO il prodotto P2

--     a) Ricerca dei codici dei fornitori che forniscono
--        prodotti diversi da P2
--        Risultato: ('F1', 'F2', 'F4')
SELECT CodF
FROM   FP
WHERE  CodP <> 'P2';

--     b) Ricerca dei fornitori che forniscono ALMENO P2
--        Risultato: ('F1', 'F2', 'F3') 
SELECT CodF
FROM   FP
WHERE  CodP = 'P2';

--     c) Ricerca dei fornitori il cui codice
--        NON sta in ('F1', 'F2', 'F4')
--        E   sta in ('F1', 'F2', 'F3')
SELECT NomeF
FROM   F
WHERE  CodF NOT IN (
          SELECT CodF
          FROM   FP
          WHERE  CodP <> 'P2'
       )
       AND CodF IN (
          SELECT CodF
          FROM   FP
          WHERE  CodP = 'P2'
       );

-- ========================================================
-- COSTRUTTORE DI TUPLA

-----------------------------------------------------------
-- 01) Selezionare il nome ed il colore dei prodotti per i  
--     quali non esistono prodotti di ugual nome e colore
--     con taglia minore o uguale a 44
SELECT NomeP, Colore
FROM   P
WHERE  (NomeP, Colore) NOT IN (
          SELECT NomeP, Colore
          FROM   P
          WHERE  Taglia <= 44
       );
-- Nota: la Camicia Blu NON  compresa nel risultato
--       poich esiste una Camicia Blu con taglia 44

-- ========================================================
-- Operatore EXISTS

-----------------------------------------------------------
-- 01) Trovare il codice dei prodotti forniti da F1

--     a) Passaggio (implicito) per ogni prodotto X:
--        ricerca se esite in FP una riga in cui
--        . CodP  il codice del prodotto X
--        . 'F2'  il codice del fornitore
--        Nota: l'istruzione seguente  solo dimostrativa!
SELECT CodP,
       (CASE WHEN EXISTS(SELECT * FROM FP
        WHERE  CodF = 'F2' AND FP.CodP = P.CodP)
        THEN 'In FP esiste fornitura di ' || CodP ||
             ' da parte di F2'
        ELSE 'In FP NON esiste fornitura di ' || CodP ||
             ' da parte di F2'
        END)
        AS Cerca_Fornitura
FROM P;

--     b) Selezione delle righe con 'Esiste fornitura'
SELECT CodP 
FROM   P
WHERE  EXISTS (
          SELECT * 
          FROM   FP
          WHERE  CodF = 'F2'
                 AND FP.CodP = P.CodP
        );

-----------------------------------------------------------
-- 02) Trovare il nome dei fornitori del prodotto P2
--     -> Trovare il nome dei fornitori PER CUI ESISTE
--        una fornitura del prodotto P2
--     -> un fornitore di codice 'Fx'  incluso nel
--        risultato finale se esiste una fornitura con
--        . codice prodotto  = 'P2'
--        . codice fornitore = 'Fx'
SELECT NomeF 
FROM   F
WHERE  EXISTS (
          SELECT * 
          FROM   FP
          WHERE  CodP = 'P2'
                 AND FP.CodF = F.CodF
        );
-- Nota: (FP.CodF = F.CodF) si chiama
--       Condizione di CORRELAZIONE
--       perch correla un attributo del livello corrente
--       ad un attributo di livello SUPERIORE
  
-- ========================================================
-- Operatore NOT EXISTS

-----------------------------------------------------------
-- 01) Trovare il codice dei prodotti NON forniti da F1

--     a) Passaggio (implicito) per ogni prodotto X:
--        ricerca se esite in FP una riga in cui
--        . CodP  il codice del prodotto X
--        . 'F2'  il codice del fornitore
--        Nota: l'istruzione seguente  solo dimostrativa!
SELECT CodP,
       (CASE WHEN EXISTS(SELECT * FROM FP
        WHERE  CodF = 'F2' AND FP.CodP = P.CodP)
        THEN 'In FP esiste fornitura di ' || CodP ||
             ' da parte di F2'
        ELSE 'In FP NON esiste fornitura di ' || CodP ||
             ' da parte di F2'
        END)
        AS Cerca_Fornitura
FROM P;

--    b) Selezione delle righe dove 'NON esiste fornitura'
SELECT CodP 
FROM   P
WHERE  NOT EXISTS (
          SELECT * 
          FROM   FP
          WHERE  CodF = 'F2'
                 AND FP.CodP = P.CodP
        );
        
-----------------------------------------------------------
-- 02) Trovare il nome dei fornitori
--     che non hanno mai fornito il prodotto P2
--     -> Trovare il nome dei fornitori PER CUI NON ESISTE
--        alcuna fornitura del prodotto P2
--     -> un fornitore di codice 'Fx'  incluso nel
--        risultato finale se NON esiste una fornitura con
--        . codice prodotto  = 'P2'
--        . codice fornitore = 'Fx'SELECT NomeF 
SELECT NomeF
FROM   F
WHERE  NOT EXISTS (
          SELECT * 
          FROM   FP
          WHERE  CodP = 'P2'
                 AND FP.CodF=F.CodF  -- <- Condizione di
       );                            --    CORRELAZIONE

-- ========================================================
-- Condizioni di CORRELAZIONE

-- . Si utilizzano in una clausola WHERE
-- . Contengono riferimenti agli attributi
--   di un'interrogazione pi esterna
-- . Non possono contenere riferimenti agli attributi
--   di un'interrogazione pi interna (nidificata)

-----------------------------------------------------------
-- 01) Per ogni prodotto, trovare il codice del fornitore
--     che ne fornisce la quantit massima
SELECT CodP, CodF 
FROM   FP
WHERE  Qta = (
          SELECT MAX(Qta)
          FROM   FP FP1
          WHERE  FP1.CodP = FP.CodP  -- <- Condizione di
       );                            --    CORRELAZIONE

-----------------------------------------------------------
-- 02) Per ogni sede di fornitore, trovare
--     nome, sede e numero soci del fornitore
--     che ha il maggior numero di soci in quella sede
SELECT NomeF, NSoci, Sede
FROM   F
WHERE  NSoci = (
          SELECT MAX(NSoci)
          FROM   F F1
          WHERE  F1.Sede = F.Sede  -- <- Condizione di
       );                          --    CORRELAZIONE
     
-- ========================================================
-- Operazione di DIVISIONE

-----------------------------------------------------------
-- 01) Trovare il codice dei fornitori
--     che forniscono TUTIi i prodotti
--     . Tutti i prodotti che possono essere forniti
--       sono contenuti nella tabella P
--       -> un fornitore fornisce tutti i prodotti
--          se fornisce un numero di prodotti diversi
--          uguale alla cardinalit di P
SELECT   CodF 
FROM     FP
GROUP BY CodF
HAVING   COUNT(*) = (
            SELECT COUNT(*)  -- <- Numero totale
            FROM P           --    di prodotti
         );
 
-----------------------------------------------------------
-- 02) Trovare il codice dei fornitori che forniscono
--     almeno tutti i prodotti forniti dal fornitore F2

--     a) Ricerca dei fornitori che forniscono
--        almeno un prodotto fornito da F2
SELECT   CodF 
FROM     FP
WHERE    CodF <> 'F2'
         AND CodP IN (           --    CodP deve stare
            SELECT CodP          -- <- tra i prodotti 
		    FROM   FP            --    forniti da F2
		    WHERE  CodF = 'F2'
         );
         
--     b) Ricerca tra i fornitori trovati in precedenza
--        selezionando quelli che hanno fornito
--        lo STESSO NUMERO di prodotti di F2
SELECT   CodF 
FROM     FP
WHERE    CodF <> 'F2'
         AND CodP IN (          --    CodP deve stare
            SELECT CodP         -- <- tra i prodotti 
		    FROM   FP           --    forniti da F2
		    WHERE  CodF = 'F2'
         )
GROUP BY CodF                    
HAVING   COUNT(*) = (           --    il numero di prodotti
            SELECT COUNT(*)     -- <- deve essere uguale
            FROM   FP           --    al numero di prodotti 
            WHERE  CodF = 'F2'  --    forniti da F2
         );
         
-- ========================================================
-- TABLE FUNCTIONS

-----------------------------------------------------------
-- 01) Per ogni magazzino di P, trovare codice e nome del
--     prodotto con la maggior quantit di pezzi forniti
--
--     a) Ricerca del totale forniture (TotQta)
--        per ogni coppia <magazzino, prodotto> 
SELECT   Magazzino, P.CodP, SUM(Qta) AS TotQta
FROM     P, FP
WHERE    P.CodP = FP.CodP
GROUP BY Magazzino, P.CodP;
--
--     b) Ricerca del massimo totale forniture (TotQta)
--        per ogni magazzino (senza indicare il prodotto)
SELECT   Magazzino, MAX(TotQta) AS MaxQta
FROM     (
            SELECT   Magazzino, P.CodP, SUM(Qta) AS TotQta
            FROM     P INNER JOIN FP ON P.CodP = FP.CodP
            GROUP BY Magazzino, P.CodP
         )
GROUP BY Magazzino;

--     c) Usando il risultato di (a), ricerca dei prodotti
--        in cui la fornitura  uguale al massimo
--        indicato in (b)
SELECT DISTINCT A.Magazzino, A.CodP, A.NomeP, A.TotQta
FROM   (
          SELECT   Magazzino, P.CodP, NomeP, SUM(Qta) AS TotQta
          FROM     P, FP
          WHERE    P.CodP = FP.CodP
          GROUP BY Magazzino, P.CodP, NomeP
       ) A,
       (
          SELECT   Magazzino, MAX(TotQta) AS MaxQta
          FROM     (
                      SELECT   Magazzino, P.CodP, SUM(Qta) AS TotQta
                      FROM     P INNER JOIN FP ON P.CodP = FP.CodP
                      GROUP BY Magazzino, P.CodP
                   )
          GROUP BY Magazzino
       ) B
WHERE A.TotQta = B.MaxQta;
