-- ========================================================
-- Operatore UNION
--
-- A UNION [ALL] B = A unione B
-- . UNION    : rimuove i duplicati nel risultato
-- . UNION ALL: non rimuove i duplicati nel risultato
-- . nota     : schemi di A, B compatibili tra loro

-----------------------------------------------------------
-- 01) Trovare il codice dei prodotti di colore rosso
--     o forniti dal fornitore F2 (o entrambe le cose)

--     a) UNION (senza duplicati)
SELECT  CodP 
FROM    P
WHERE   Colore = 'Rosso'
UNION
SELECT  CodP 
FROM    FP
WHERE   CodF = 'F2';

--     b) UNION ALL (con duplicati)
SELECT  CodP 
FROM    P
WHERE   Colore = 'Rosso'
UNION ALL
SELECT  CodP 
FROM    FP
WHERE   CodF = 'F2';

--     c) soluzione senza UNION
SELECT  DISTINCT P.CodP 
        FROM P, FP
WHERE   P.CodP = FP.CodP
        AND (Colore = 'Rosso' OR CodF = 'F2');
        
-----------------------------------------------------------
-- 02) Trovare il codice dei prodotti forniti con Qta > 300
--     nell'anno corrente e/o nell'anno precedente
--     (tabella FPP: forniture dell'anno precedente)

--     a) con una clausola WHERE per ogni tabella 
SELECT  CodP
FROM    FP
WHERE   Qta > 300
UNION
SELECT  CodP
FROM    FPP
WHERE   Qta > 300;

--     b) con una clausola WHERE unica 
SELECT  DISTINCT CodP
FROM    (
            SELECT  CodP, Qta
            FROM    FP
            UNION
            SELECT  CodP, Qta
            FROM    FPP
        )
WHERE   Qta > 300;

--     c) con un attributo che indica l'anno 
SELECT  CodP, 'Corrente' AS Anno
FROM    FP
WHERE   Qta > 300
UNION
SELECT  CodP, 'Precedente'
FROM    FPP
WHERE   Qta > 300
ORDER BY Anno, CodP;

-----------------------------------------------------------
-- 03) Trovare, per ciascun fornitore
--     . una riga con il totale dei pezzi forniti
--       nell'anno corrente e l'indicazione 'Corrente' 
--     . una riga con il totale dei pezzi forniti
--       nell'anno precedente e l'indicazione 'Precedente' 
--     (tabella FPP: forniture dell'anno precedente)

--     a) Soluzione NON corretta (UNION senza ALL)
SELECT  CodF, SUM(Qta), Anno
FROM    (
            SELECT  CodF, Qta, 'Corrente' AS Anno
            FROM    FP
            UNION
            SELECT  CodF, Qta, 'Precedente' AS Anno
            FROM    FPP
        ) FPT
GROUP   BY CodF, Anno 
ORDER   BY Anno, CodF;

--     b) Soluzione corretta (UNION ALL)
SELECT  CodF, SUM(Qta), Anno
FROM    (
            SELECT  CodF, Qta, 'Corrente' AS Anno
            FROM    FP
            UNION ALL
            SELECT  CodF, Qta, 'Precedente' AS Anno
            FROM    FPP
        ) FPT
GROUP   BY CodF, Anno
ORDER   BY Anno, CodF;

-- ========================================================
-- Operatore INTERSECT
--
-- A INTERSECT B = A intersezione B
-- . INTERSECT: rimuove i duplicati nel risultato
-- . nota     : schemi di A, B compatibili tra loro
     
-----------------------------------------------------------
-- 01) Trovare le citt che sono sia sede di fornitori,
--     sia magazzino di prodotti

--     a) Soluzione con INTERSECT 
SELECT  Sede 
FROM    F
INTERSECT
SELECT  Magazzino 
FROM    P;

--     b) Soluzione con IN
SELECT  DISTINCT Magazzino 
FROM    P
WHERE   Magazzino IN (
            SELECT  Sede 
            FROM    F
        );

--     c) Soluzione con JOIN
SELECT  DISTINCT Magazzino 
FROM    F, P
WHERE   F.Sede = P.Magazzino;

-----------------------------------------------------------
-- 02) Trovare le coppie <CodF, CodP> per le quali
--      stata fornita la stessa quantit di pezzi 
--     nell'anno corrente e nell'anno precedente

--     a) con indicazione della quantit
SELECT  *
FROM    FP
INTERSECT
SELECT  *
FROM    FPP;

--     b) senza indicazione della quantit
SELECT  CodF, CodP
FROM    (
            SELECT  *
            FROM    FP
            INTERSECT
            SELECT  *
            FROM    FPP
        );
        
-----------------------------------------------------------
-- 03) Trovare le coppie <NomeF, NomeP> per le quali
--      stata fornita la stessa quantit di pezzi 
--     nell'anno corrente e nell'anno precedente
SELECT  NomeF, NomeP
FROM    F, P, (
            SELECT  *
            FROM    FP
            INTERSECT
            SELECT  *
            FROM    FPP
        ) FPT 
WHERE   F.CodF = FPT.CodF
        AND P.CodP = FPT.CodP;
        
-- ========================================================
-- Operatore MINUS (anche EXCEPT da Oracle 20c)
--
-- A MINUS B = A meno B
-- . MINUS: rimuove i duplicati nel risultato
-- . nota : schemi di A, B compatibili tra loro

-----------------------------------------------------------
-- 01) Trovare le citt che sono sede di fornitori,
--     ma non magazzino di prodotti

--     a) Soluzione con MINUS
SELECT  Sede 
FROM    F
MINUS
SELECT  Magazzino 
FROM    P;

--     b) Soluzione con NOT IN
SELECT  Sede 
FROM    F
WHERE   Sede NOT IN (
            SELECT  Magazzino 
            FROM P
        );

-----------------------------------------------------------
-- 02) Trovare le sedi dei fornitori
--     che non stanno a Milano

--     a) Soluzione con MINUS
SELECT  Sede 
FROM    F
MINUS
SELECT  'Milano'
FROM    F;

--     b) Soluzione con WHERE
SELECT  DISTINCT Sede 
FROM    F
WHERE   Sede <>  'Milano';