-- ========================================================
-- INSERT

-----------------------------------------------------------
-- 01) Inserire un prodotto con
--     CodP = 'P7', NomeP = 'Maglia', Colore = 'Viola',
--     Taglia = 40, Citt = 'Venezia'

--     a) Con indicazione degli attributi
INSERT	INTO P (CodP, NomeP, Colore, Taglia, Citt)
VALUES 	('P7', 'Maglia', 'Viola', 40, 'Venezia');

--     b) Senza indicazione degli attributi
--        (quando si specificano tutti i valori)
INSERT	INTO P
VALUES 	('P7', 'Maglia', 'Viola', 40, 'Venezia');

-----------------------------------------------------------
-- 02) Inserire un prodotto con
--     CodP = 'P8', Taglia = 42, Citt = 'Genova'

INSERT	INTO P (CodP, Citt, Taglia)
VALUES	('P8', 'Genova', 42);

-- Nota: gli attributi non specificati assumono valore NULL
--       e devono quindi comprendere NULL nel dominio

-----------------------------------------------------------
-- 03) Inserire una nuova fornitura con
--     CodF = 'F2', CodP = 'P3', Qta = 800

INSERT	INTO FP (CodF, CodP, Qta)
VALUES 	('F2', 'P3', 800);

-----------------------------------------------------------
-- 04) Inserire una nuova fornitura con
--     CodF = 'F1', CodP = 'P1', Qta = 800

INSERT	INTO FP (CodF, CodP, Qta)
VALUES 	('F1', 'P1', 800);

-- Nota: l'operazione non viene eseguita in quanto 
--       (CodF, CodP)  la chiave primaria della tabella FP
--       e in FP esiste gi una fornitura con questi dati

-----------------------------------------------------------
-- 05) Inserire una nuova fornitura con
--     CodF = 'F20', CodP = 'P20', Qta = 1000

INSERT	INTO FP (CodF, CodP, Qta)
VALUES 	('F20', 'P20', 1000);

-- Nota: se sono attivi i vincoli di integrit referenziale
--       e non esiste il fornitore F20 o il prodotto P20,
--       l'operazione non viene eseguita

-----------------------------------------------------------
-- 06) Inserire, per il fornitore F5, le stesse forniture
--     relative al fornitore F1

INSERT	INTO FP (CodF, CodP, Qta)
SELECT	'F5', CodP, Qta
WHERE	CodF = 'F1';

-- ========================================================
-- DELETE
--
-- Nota: nel DB Fornitori-Prodotti sono impostati
--       i seguenti vincoli di integrit referenziale:
--       FP.CodF -> F.CodF (nessuna azione compensativa)
--       FP.CodP -> P.CodP (nessuna azione compensativa)

-----------------------------------------------------------
-- 01) Cancellare tutte le forniture dell'anno precedente

DELETE	FROM FPP;

-- Nota: senza clausola WHERE tutte le tuple di FPP
--       soddisfano il predicato di selezione

-----------------------------------------------------------
-- 02) Cancellare in FP le forniture di F5
DELETE	FROM FP 
WHERE	CodF = 'F5';

-----------------------------------------------------------
-- 03) Cancellare in F i dati del fornitore F5
DELETE	FROM F 
WHERE	CodF = 'F5';

-- Nota: l'operazione viene svolta regolarmente in quanto
--       non viola il vincolo di integrit referenziale
--          FP.CodF -> F.CodF 
--       (le forniture di F5 sono state eliminate)

-----------------------------------------------------------
-- 04) Cancellare in F i dati del fornitore F1
DELETE	FROM F 
WHERE	CodF = 'F1';

-- Nota: l'operazione non viene svolta in quanto violerebbe
--       il vincolo di integrit FP.CodF -> F.CodF
--       (esistono prodotti forniti da F1)
 
 -- ========================================================
-- UPDATE
--
-- Nota: nel DB Fornitori-Prodotti sono impostati
--       i seguenti vincoli di integrit referenziale:
--       FP.CodF -> F.CodF (nessuna azione compensativa)
--       FP.CodP -> P.CodP (nessuna azione compensativa)

-----------------------------------------------------------
-- 01) Aggiornare le caratteristiche del prodotto P1:
--        assegnare Giallo al Colore
--        incrementare la Taglia di 2
--        assegnare NULL a Citt 

UPDATE	P 
SET		Colore = 'Giallo',  
		Taglia = Taglia + 2, 
		Citt = NULL
WHERE 	CodP = 'P1';

-----------------------------------------------------------
-- 02) Aggiornare il numero dei soci al doppio del valore
--     per tutti i fornitori con sede a Milano

UPDATE	F
SET		Nsoci = 2 * NSoci
WHERE	Citt = 'Milano';

-----------------------------------------------------------
-- 03) Assegnare 10 alla quantit fornita per tutti
--     i fornitori di Milano
UPDATE	FP 
SET		Qta = 10
WHERE	CodF IN (
			SELECT 	CodF 
			FROM 	F
			WHERE 	Sede = 'Milano'
		);

-----------------------------------------------------------
-- 04) Per ogni fornitura in FP, assegnare alla quantit
--     di pezzi fornita quest'anno lo stesso valore della
--     fornitura corrispondente in FPP (se in FPP esiste
--     una fornitura con gli stessi valori di CodF e CodP)

UPDATE	FP
SET		Qta = (
            SELECT  Qta
            FROM    FPP
            WHERE	FP.CodF = FPP.CodF
            AND FP.CodP = FPP.CodP
        );

-----------------------------------------------------------
-- 05) Assegnare il valore CodF = F9
--     al fornitore avente CodF = F2

UPDATE	F
SET		CodF = 'F9'
WHERE	CodF = 'F2';

-- Nota: l'operazione non viene svolta in quanto
--       in FP esistono forniture effettuate da F2
--       e nel DB non  prevista alcuna azione compensativa
--       per la 'scomparsa' di F2 dalla tabella F




