-- ========================================================
-- CREATE TABLE

-----------------------------------------------------------
-- Script per la creazione del DB Fornitori-Prodotti
-- (tabelle F, P, FP)

-- Tabella Fornitori
CREATE TABLE F (
    -- vincoli su CodF e NomeF
	CodF CHAR(2) NOT NULL,
	NomeF VARCHAR(10) NOT NULL,
	NSoci SMALLINT,
    Sede VARCHAR(10),
	PRIMARY KEY(CodF)
);

-- Tabella Prodotti
CREATE TABLE P (
    -- vincoli su CodP e NomeP
	CodP CHAR(2) NOT NULL,
	NomeP VARCHAR(10) NOT NULL,
	Colore VARCHAR(10),
	Taglia SMALLINT,
	Magazzino VARCHAR(10),
	PRIMARY KEY (CodP)
);

-- Tabella Forniture
CREATE TABLE FP (
	CodF CHAR(2) NOT NULL,
	CodP CHAR(2) NOT NULL,
	-- vincolo su Qta - nome del vincolo: cv_Qta
	Qta SMALLINT
	CONSTRAINT cv_Qta CHECK (Qta IS NOT NULL AND Qta > 0),
	PRIMARY KEY (CodF, CodP),
	-- clausola di integrit referenziale CodF -> F.CodF
	-- (senza azioni compensative) -  nome: fk_CodF
	CONSTRAINT fk_CodF
	FOREIGN KEY(CodF)
		REFERENCES F(CodF),
	-- clausola di integrit referenziale CodP -> P.CodP
	-- (senza azioni compensative) -  nome: fk_CodP
	CONSTRAINT fk_CodP
	FOREIGN KEY(CodP)
		REFERENCES P(CodP)
);

-----------------------------------------------------------
-- 01) Modificare le clausole di integrit referenziale
--     in FP in modo che la cancellazione di un prodotto
--     comporti la cancellazione in cascata
--     di tutte le sue forniture in FP

-- elimina il vincolo preesistente fk_CodP
ALTER TABLE FP DROP CONSTRAINT fk_CodP;

-- definisce il nuovo vincolo fk_CodP
ALTER TABLE FP ADD CONSTRAINT fk_CodP
	FOREIGN KEY(CodP)
		REFERENCES P(CodP)
		ON DELETE CASCADE
		ENABLE;
		
-----------------------------------------------------------
-- 02) Eliminare il prodotto di codice P2
--     -> sono eliminate tutte le forniture di P2 in FP

DELETE
FROM 	P
WHERE	CodP = 'P2';

-----------------------------------------------------------
-- 03) Modificare le clausole di integrit referenziale
--     in FP in modo che la modifica di un codice fornitore
--     comporti la modifica in cascata
--     di tutte le sue forniture in FP

-- elimina il vincolo preesistente fk_CodF
ALTER TABLE FP DROP CONSTRAINT fk_CodF;

-- definisce il nuovo vincolo fk_CodF
-- Nota: in Oracle la clausola ON UPDATE CASCADE non esiste 
ALTER TABLE FP ADD CONSTRAINT fk_CodF
	FOREIGN KEY(CodF)
		REFERENCES F(CodF)
		ON UPDATE CASCADE
		ENABLE;
		
-----------------------------------------------------------
-- 04) Modificare il codice del fornitore F1 in F9
--     -> sono modificate tutte le forniture di F1 in FP
--        (non in Oracle!)

UPDATE	F
SET 	CodF = 'F9'
WHERE	CodF = 'F1';

-----------------------------------------------------------
-- 05) Modificare il codice del fornitore F1 in F9
--     e tutte le corrispondenti forniture di F1 in FP
--     -> in Oracle

-- disabilita il vincolo fk_CodF
ALTER TABLE FP
DISABLE CONSTRAINT fk_CodF;

-- in F modifica F1 in F9
UPDATE	F
SET 	CodF = 'F9'
WHERE	CodF = 'F1';

-- in FP modifica tutte le occorrenze F1 in F9
UPDATE	FP
SET 	CodF = 'F9'
WHERE	CodF = 'F1';

-- riabilita il vincolo fk_CodF
ALTER TABLE FP
ENABLE CONSTRAINT fk_CodF;